#!/bin/ash

###	But I'm sorry, follows comments are Japanese-English, you may not understand it.
###	This script does backup without rotate.

###
#	Input
#	$1 : Guest name
#	$2 : $gid = Guest ID
#
#	Internal variable
#	$gname	: Guest name
#	$gid	: Guest ID
#	$store	: Gust datastore name
#	$gdir	: Gust datastore directory
#	$bdate	: Backup date
#	$bbase	: Backup base directory(/vmfs/volumes/esx01-backup/)
#	$bdist	: Distination Backup path(/vmfs/volumes/vmdatabackup/)
#	$bdir	: Backup directory
#
#	07/05/2009 by hiro
#	07/07/2009 add and fix by hiro
#	05/17/2011 modified by KamiRa. http://nakoruru.jp/
###

###-----
#	wait current snapshot creation work
#
check_running_task (){
for i in `vim-cmd vimsvc/task_list | grep vim.Task:haTask-$1 | grep $2 | sed -e 's/.*vim.Task://' -e "s/[', ]//g"`; do
if [ "`vim-cmd vimsvc/task_info $i | grep running`" != "" ] ; then
	return 0
fi
done
return 1
}
#
###-----

###=======================
#
#	Start main
#
###=======================

# the backup root directory
bbase="/vmfs/volumes/esx01-backup/"
bdist="/vmfs/volumes/vmdatabackup/"

#bdate=`date +%Y%m%d`
#echo $bdate

# the Guest name and get the Guest ID
# -------------------------------------------------------- 07/07/2009 added by hiro
#   Check parameter
if test -z $1
then
        exit 1
fi
gname=$1
gid=`vim-cmd vmsvc/getallvms | grep " $gname " | awk '{ print $1 }'`
# -------------------------------------------------------- 07/07/2009 fixed by hiro
#   Changed how to check GID
# if [ $? -eq 0 ]; then
if test -z $gid
then
	gid=`vim-cmd vmsvc/getallvms | grep $gname | awk '{ print $1 }'`
fi

if test -z $gid
then
	echo "Error : Guest - "$gname" did not exist."
	exit 0
fi
echo "vmid: $gid"
                

# take store location of the Guest and set full directory path of the Guest VM files
store=`vim-cmd vmsvc/getallvms | grep $gname | grep $gid | sed -e 's/\[//g' -e 's/\]//g' | awk '{ print $3 }'`
gdir="/vmfs/volumes/"$store"/"$gname"/"
echo "source path: $gdir"

# set backup directory path
bdir=$bdist$gname"/"
if test ! -d $bdir
then
	mkdir $bdir
else
	echo "Already exist distination backup path."
fi

###-----
#	Start backup
#
#---(Part.1)	copy VM managed information files, you have to copy these informations before make a clone.
#
vmxpath=`ls $gdir | grep ".vmx" | grep -v ".vmxf"`
vmxfpath=`ls $gdir | grep ".vmxf"`
vmsdpath=`ls $gdir | grep ".vmsd"`
#vmdkpath="$gname.vmdk" ;  vmdkpath=`ls $gdir | grep $vmdkpath`
rm -rf tmp ; vmdkpath=`ls $gdir | grep ".vmdk" | grep -v "delta" | grep -v "flat" >> tmp`

while read LINE; do
	echo $LINE
done < "tmp"

echo "$gdir$vmxpath --> $bdir" ; cp $gdir$vmxpath $bdir
echo "$gdir$vmxfpath --> $bdir" ; cp $gdir$vmxfpath $bdir
echo "$gdir$vmsdpath --> $bdir" ; cp $gdir$vmsdpath $bdir

#
#---(Part.2)	take a snapshot
#
vim-cmd vmsvc/snapshot.create $gid $gname
if [ $? -ne 0 ]; then
	echo Snapshot create error.;
	exit;
fi
while check_running_task $gid createSnapshot; do sleep 1; done

#
#---(Part.3)	cloning with monosparse, if you want flat image, remove option.
#
while read LINE; do
	vmdkpath=$LINE
	echo "$gdir$vmdkpath --> $bdir$vmdkpath"
	vmkfstools -i $gdir$vmdkpath -d monosparse $bdir$vmdkpath
	if [ $? -ne 0 ]; then
		echo "Error : cloning failed. your backup process did not complete."
		exit 0
	fi
done < "tmp"
rm -rf tmp

#
#---(Part.4)	remove a snapshot
#
vim-cmd vmsvc/snapshot.removeall $gid
#
#	Backup finished

echo "Congratulations!  Guest-"$gname" : backup completely successed."
exit 0
#
###-----
